
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.util.Random;
import java.util.concurrent.PriorityBlockingQueue;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

// Airplane class
class Airplane extends Thread implements Comparable<Airplane> {
  private final String airplaneId;
  private final int fuelLevel;
  private final boolean isLanding;
  private final ATC atc;

  public Airplane(String airplaneId, int fuelLevel, boolean isLanding, ATC atc) {
    this.airplaneId = airplaneId;
    this.fuelLevel = fuelLevel;
    this.isLanding = isLanding;
    this.atc = atc;
  }

  public String getAirplaneId() {
    return airplaneId;
  }

  public int getFuelLevel() {
    return fuelLevel;
  }

  public boolean isLanding() {
    return isLanding;
  }

  @Override
  public void run() {
    if (isLanding) {
      atc.requestLanding(this);
    } else {
      atc.requestTakeoff(this);
    }
  }

  @Override
  public int compareTo(Airplane other) {
    return Integer.compare(this.fuelLevel, other.fuelLevel);
  }
}

// LandingStrip class
class LandingStrip {
  private final int id;
  private boolean isAvailable = true;

  public LandingStrip(int id) {
    this.id = id;
  }

  public int getId() {
    return id;
  }

  public synchronized boolean useStrip(Airplane airplane, ATCFrame frame) {
    if (isAvailable) {
      isAvailable = false;
      frame.updateStripStatus(id, airplane.getAirplaneId(), "Volando");
      try {
        Thread.sleep(10000); // Simulate time to land or takeoff
      } catch (InterruptedException e) {
        Thread.currentThread().interrupt();
      }
      frame.updateStripStatus(id, null, "Disponible");
      isAvailable = true;
      return true;
    }
    return false;
  }

  public boolean isAvailable() {
    return isAvailable;
  }
}

// ATC class
class ATC {
  private final PriorityBlockingQueue<Airplane> landingQueue = new PriorityBlockingQueue<>();
  private final LandingStrip[] landingStrips;
  private final Lock lock = new ReentrantLock();
  private final ATCFrame frame;

  public ATC(int numberOfStrips, ATCFrame frame) {
    this.frame = frame;
    landingStrips = new LandingStrip[numberOfStrips];
    for (int i = 0; i < numberOfStrips; i++) {
      landingStrips[i] = new LandingStrip(i + 1);
    }

    // Background thread to continuously assign strips
    new Thread(() -> {
      while (true) {
        assignStrip();
        try {
          Thread.sleep(500); // Check periodically
        } catch (InterruptedException e) {
          Thread.currentThread().interrupt();
        }
      }
    }).start();
  }

  public void requestLanding(Airplane airplane) {
    frame.addAirplaneToRegister(airplane, "Esperando");
    landingQueue.add(airplane);
    frame.updateWaitingQueue(landingQueue);
  }

  public void requestTakeoff(Airplane airplane) {
    frame.addAirplaneToRegister(airplane, "Esperando");
    landingQueue.add(airplane);
    frame.updateWaitingQueue(landingQueue);
  }

  private void assignStrip() {
    lock.lock();
    try {
      for (LandingStrip strip : landingStrips) {
        if (!landingQueue.isEmpty() && strip.isAvailable()) {
          Airplane airplane = landingQueue.poll();
          frame.updateWaitingQueue(landingQueue); // Update waiting queue
          frame.updateAirplaneStatus(airplane.getAirplaneId(), "Asignado a la pista " + strip.getId());
          new Thread(() -> strip.useStrip(airplane, frame)).start();
        }
      }
    } finally {
      lock.unlock();
    }
  }

}

// ATC GUI
class ATCFrame extends JFrame {
  private final DefaultTableModel airplaneTableModel;
  private final DefaultTableModel stripTableModel;
  private final DefaultListModel<String> waitingQueueModel; // Model for the waiting queue

  public ATCFrame() {
    setTitle("Control de tráfico aéreo");
    setSize(800, 600);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setLayout(new BorderLayout(10, 10));

    // Panels for different sections
    JPanel registerPanel = new JPanel(new BorderLayout());
    registerPanel.setBorder(BorderFactory.createTitledBorder("Registro de aviones"));

    JPanel stripPanel = new JPanel(new BorderLayout());
    stripPanel.setBorder(BorderFactory.createTitledBorder("Estado de la pista"));

    JPanel queuePanel = new JPanel(new BorderLayout());
    queuePanel.setBorder(BorderFactory.createTitledBorder("Esperando usar la pista"));

    // Airplane Register Table
    airplaneTableModel = new DefaultTableModel(new String[] { "Avión ID", "Combustible", "Estado" }, 0);
    JTable airplaneTable = new JTable(airplaneTableModel);
    JScrollPane airplaneScrollPane = new JScrollPane(airplaneTable);
    registerPanel.add(airplaneScrollPane, BorderLayout.CENTER);

    // Landing Strip Status Table
    stripTableModel = new DefaultTableModel(new String[] { "Pista de aterrizaje", "Avión actual", "Estado" }, 0);
    JTable stripTable = new JTable(stripTableModel);
    JScrollPane stripScrollPane = new JScrollPane(stripTable);
    stripPanel.add(stripScrollPane, BorderLayout.CENTER);

    // Waiting Queue List
    waitingQueueModel = new DefaultListModel<>();
    JList<String> waitingQueueList = new JList<>(waitingQueueModel);
    JScrollPane queueScrollPane = new JScrollPane(waitingQueueList);
    queuePanel.add(queueScrollPane, BorderLayout.CENTER);

    // Adding panels to the frame
    add(registerPanel, BorderLayout.CENTER); // Center
    add(stripPanel, BorderLayout.SOUTH); // Bottom
    add(queuePanel, BorderLayout.EAST); // Right side
  }

  public void addAirplaneToRegister(Airplane airplane, String status) {
    SwingUtilities.invokeLater(() -> airplaneTableModel
        .addRow(new Object[] { airplane.getAirplaneId(), airplane.getFuelLevel(), status }));
  }

  public void updateAirplaneStatus(String airplaneId, String status) {
    SwingUtilities.invokeLater(() -> {
      for (int i = 0; i < airplaneTableModel.getRowCount(); i++) {
        if (airplaneTableModel.getValueAt(i, 0).equals(airplaneId)) {
          airplaneTableModel.setValueAt(status, i, 2);
          break;
        }
      }
    });
  }

  public void updateStripStatus(int stripId, String airplaneId, String status) {
    SwingUtilities.invokeLater(() -> {
      if (stripTableModel.getRowCount() < stripId) {
        stripTableModel.addRow(new Object[] { stripId, airplaneId, status });
      } else {
        stripTableModel.setValueAt(airplaneId, stripId - 1, 1);
        stripTableModel.setValueAt(status, stripId - 1, 2);
      }
    });
  }

  public void updateWaitingQueue(PriorityBlockingQueue<Airplane> queue) {
    SwingUtilities.invokeLater(() -> {
      waitingQueueModel.clear();
      for (Airplane airplane : queue) {
        waitingQueueModel.addElement("ID: " + airplane.getAirplaneId() + ", Combustible: " + airplane.getFuelLevel());
      }
    });
  }
}

// Main Class
public class AirTrafficControlSimulation {
  public static void main(String[] args) {
    ATCFrame frame = new ATCFrame();
    frame.setVisible(true);

    ATC atc = new ATC(3, frame); // 3 landing strips

    // Generate airplanes dynamically
    new Thread(() -> {
      Random random = new Random();
      int airplaneCounter = 1;

      while (true) {
        boolean isLanding = random.nextBoolean();
        int fuelLevel = random.nextInt(100) + 1;
        String airplaneId = "A" + airplaneCounter++;

        Airplane airplane = new Airplane(airplaneId, fuelLevel, isLanding, atc);
        airplane.start();

        try {
          Thread.sleep(random.nextInt(2000) + 1000); // New airplane every 1-3 seconds
        } catch (InterruptedException e) {
          Thread.currentThread().interrupt();
        }
      }
    }).start();
  }
}
